@@ -60,6 +60,9 @@ setMethod("ext", signature(x="numeric"),
 
 setMethod("ext", signature(x="matrix"),
 	function(x){
+		if ((ncol(x) == 2) && (nrow(x) > 2)) {
+			x <- apply(x, 2, range, na.rm=TRUE)
+		}
 		ext(as.vector(x))
 	}
 )
@@ -71,9 +74,6 @@ setMethod("ext", signature(x="bbox"),
 )
 
 
-
-
-
 setMethod("ext", signature(x="SpatRaster"),
 	function(x, cells=NULL){
 		if (!is.null(cells)) {
