@@ -243,7 +243,7 @@ extract_fun <- function(x, y, fun, ID=TRUE, weights=FALSE, exact=FALSE, touches=
 	} 
 	if (bind) {
 		if (nrow(e) == nrow(y)) {
-			e <- cbind(y, e[,-1,drop=FALSE])
+			e <- cbind(y, e)
 		} else {
 			warn("extract", "cannot return a SpatVector because the number of records extracted does not match he number of rows in y (perhaps you need to use a summarizing function")
 		}
@@ -289,9 +289,7 @@ function(x, y, fun=NULL, method="simple", cells=FALSE, xy=FALSE, ID=TRUE, weight
 			exact <- FALSE
 		} 
 		# method <- match.arg(tolower(method), c("simple", "bilinear"))
-	}
-
-	if (!is.null(fun)) {
+	} else if (!is.null(fun)) { # nothing to summarize for points
 		txtfun <- .makeTextFun(fun)
 		if (inherits(txtfun, "character")) {
 			if (txtfun == "table") {
