@@ -75,6 +75,33 @@ setMethod("set.names", signature(x="SpatRaster"),
 )
 
 
+setMethod("names", signature(x="SpatRasterCollection"),
+	function(x) {
+		nms <- x@ptr$names
+		Encoding(nms) <- "UTF-8"
+		nms
+	}
+)
+
+
+setMethod("names<-", signature(x="SpatRasterCollection"),
+	function(x, value) {
+		x@ptr <- x@ptr$deepcopy()
+		x@ptr$names <- enc2utf8(as.character(value))
+		x
+	}
+)
+
+setMethod("set.names", signature(x="SpatRasterCollection"),
+	function(x, value, index=1:length(x), validate=FALSE)  {
+		value <- .raster_names_check(x, value, index, validate)
+		x@ptr$names <- value
+		invisible(TRUE)
+	}
+)
+
+
+
 setMethod("names", signature(x="SpatRasterDataset"),
 	function(x) {
 		nms <- x@ptr$names
