@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2022  Robert J. Hijmans
+// Copyright (c) 2018-2023  Robert J. Hijmans
 //
 // This file is part of the "spat" library.
 //
@@ -171,7 +171,7 @@ bool differentFilenames(std::vector<std::string> inf, std::vector<std::string> o
 	#endif
 
 	for (size_t i=0; i<inf.size(); i++) {
-		if (inf[i] == "") continue;
+		if (inf[i].empty()) continue;
 		#ifdef _WIN32
 		std::transform(inf[i].begin(), inf[i].end(), inf[i].begin(), ::tolower);
 		#endif
@@ -191,8 +191,6 @@ bool differentFilenames(std::vector<std::string> inf, std::vector<std::string> o
 	return true;
 }
 
-#include "Rcpp.h"
-
 bool can_write(std::vector<std::string> filenames, std::vector<std::string> srcnames, bool overwrite, std::string &msg) {
 
 	if (!differentFilenames(srcnames, filenames, msg)) {
@@ -200,7 +198,7 @@ bool can_write(std::vector<std::string> filenames, std::vector<std::string> srcn
 	}
 
 	for (size_t i=0; i<filenames.size(); i++) {
-		if ((filenames[i] != "") && file_exists(filenames[i])) {
+		if (!filenames[i].empty() && file_exists(filenames[i])) {
 			if (overwrite) {
 				if (remove(filenames[i].c_str()) != 0) {
 					msg = ("cannot overwrite existing file");
