@@ -9,52 +9,56 @@
 
 //#include "spatFactor.h"
 #include "recycle.h"
+#include "sort.h"
 #include "gdalio.h"
 
 
-SpatRaster SpatRaster::rasterizePoints(SpatVector x, std::string fun, std::vector<double> values, double background, SpatOptions &opt) {
+SpatRaster SpatRaster::rasterizePoints(std::vector<double>&x, std::vector<double> &y, std::string fun, std::vector<double> &values, bool narm, double background, SpatOptions &opt) {
 
 	SpatRaster out = geometry(1, false, false, false);
+	
 	if (!out.writeStart(opt, filenames())) {
 		return out;
 	}
-	if (fun != "count" && (values.size() != x.size())) {
-		out.setError("values do not match geometries");
+	
+	if ((fun == "count") && (values.size() != x.size()) && (!values.empty())) {
+		out.setError("number of values does not match the number of geometries");
+		return out;
+	} else if (values.size() != x.size()) {
+		out.setError("number of values does not match the number of geometries");
 		return out;
 	}
 
 	size_t nc = ncol();
-	std::vector<std::vector<double>> pxy = x.coordinates();
-	std::vector<double> cells = cellFromXY(pxy[0], pxy[1]);
+	std::vector<double> cells = cellFromXY(x, y, -9);
+	// order for multiple chunks, but also to remove NAs (-9)
+	std::vector<std::size_t> so = sort_order_a(cells);
+	permute(cells, so);
+	permute(values, so);
+
+	size_t cellcnt = 0;
+	for (size_t i=0; i<cells.size(); i++) {
+		if (cells[i] < 0) {
+			cellcnt++;
+		} else {
+			break;
+		}
+	}
+
 	if (fun == "count") {
+		if (values.empty()) narm=false;
 		for (size_t i=0; i < out.bs.n; i++) {
 			double cmin = out.bs.row[i] * nc;
 			double cmax = (out.bs.row[i]+out.bs.nrows[i]) * nc - 1;
 			std::vector<double> v(out.bs.nrows[i] * out.ncol(), 0);
-			for (size_t j=0; j<cells.size(); j++) {
-				if (cells[j] >= cmin && cells[j] <= cmax) {
+			for (size_t j=cellcnt; j<cells.size(); j++) {
+				if (narm && std::isnan(values[j])) continue;
+				if (cells[j] <= cmax) {
 					size_t k = cells[j] - cmin;
 					v[k]++;
-				}
-			}
-			if (background != 0) {
-				for (size_t j=0; j<v.size(); j++) {
-					if (v[j] == 0) {
-						v[j] = background;
-					}
-				}
-			}
-			if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i]))  return out;
-		}
-	} else if (fun == "sum") {
-		for (size_t i=0; i < out.bs.n; i++) {
-			double cmin = out.bs.row[i] * nc;
-			double cmax = (out.bs.row[i]+out.bs.nrows[i]) * nc - 1;
-			std::vector<double> v(out.bs.nrows[i] * out.ncol(), 0);
-			for (size_t j=0; j<cells.size(); j++) {
-				if (cells[j] >= cmin && cells[j] <= cmax) {
-					size_t k = cells[j] - cmin;
-					v[k] += values[j];
+				} else {
+					cellcnt = j;
+					break;
 				}
 			}
 			if (background != 0) {
@@ -72,24 +76,162 @@ SpatRaster SpatRaster::rasterizePoints(SpatVector x, std::string fun, std::vecto
 			double cmax = (out.bs.row[i]+out.bs.nrows[i]) * nc - 1;
 			std::vector<double> v(out.bs.nrows[i] * out.ncol(), 0);
 			std::vector<double> cnt = v;
-			for (size_t j=0; j<cells.size(); j++) {
-				if (cells[j] >= cmin && cells[j] <= cmax) {
+			for (size_t j=cellcnt; j<cells.size(); j++) {
+				if (narm && std::isnan(values[j])) continue;
+				if (cells[j] <= cmax) {
 					size_t k = cells[j] - cmin;
 					v[k] += values[j];
 					cnt[k]++;
-				}
-			}
-			for (size_t j=0; j<cnt.size(); j++) {
-				if (cnt[j] > 0) {
-					v[j] /= cnt[j];
+				} else {
+					cellcnt = j;
+					break;
 				}
 			}
 			if (background != 0) {
 				for (size_t j=0; j<v.size(); j++) {
-					if (v[j] == 0) {
+					if (cnt[j] == 0) {
 						v[j] = background;
+					} else {
+						v[j] /= cnt[j];						
 					}
 				}
+			} else {
+				for (size_t j=0; j<cnt.size(); j++) {
+					if (cnt[j] > 0) {
+						v[j] /= cnt[j];
+					}
+				}				
+			}			
+			
+			if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i]))  return out;
+		}
+	} else if (fun == "sum") {
+		for (size_t i=0; i < out.bs.n; i++) {
+			double cmin = out.bs.row[i] * nc;
+			double cmax = (out.bs.row[i]+out.bs.nrows[i]) * nc - 1;
+			std::vector<double> v(out.bs.nrows[i] * out.ncol(), background);
+			std::vector<bool> newcell(out.bs.nrows[i] * out.ncol(), true);
+			for (size_t j=cellcnt; j<cells.size(); j++) {
+				if (narm && std::isnan(values[j])) continue;
+				if (cells[j] <= cmax) {
+					size_t k = cells[j] - cmin;
+					if (newcell[k]) {
+						v[k] = values[j];
+						newcell[k] = false;
+					} else {
+						v[k] += values[j];
+					}
+				} else {
+					cellcnt = j;
+					break;
+				}
+			}
+			if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i]))  return out;
+		}
+	} else if (fun == "min") {
+		for (size_t i=0; i < out.bs.n; i++) {
+			double cmin = out.bs.row[i] * nc;
+			double cmax = (out.bs.row[i]+out.bs.nrows[i]) * nc - 1;
+			std::vector<double> v(out.bs.nrows[i] * out.ncol(), background);
+			std::vector<bool> newcell(out.bs.nrows[i] * out.ncol(), true);
+			for (size_t j=cellcnt; j<cells.size(); j++) {
+				if (narm && std::isnan(values[j])) continue;
+				if (cells[j] <= cmax) {
+					size_t k = cells[j] - cmin;
+					if (newcell[k]) {
+						v[k] = values[j];
+						newcell[k] = false;
+					} else {
+						v[k] = std::min(v[k], values[j]);
+					}
+				} else {
+					cellcnt = j;
+					break;
+				}
+			}
+			if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i]))  return out;
+		}
+	} else if (fun == "max") {
+		for (size_t i=0; i < out.bs.n; i++) {
+			double cmin = out.bs.row[i] * nc;
+			double cmax = (out.bs.row[i]+out.bs.nrows[i]) * nc - 1;
+			std::vector<double> v(out.bs.nrows[i] * out.ncol(), background);
+			std::vector<bool> newcell(out.bs.nrows[i] * out.ncol(), true);
+			for (size_t j=cellcnt; j<cells.size(); j++) {
+				if (narm && std::isnan(values[j])) continue;
+				if (cells[j] <= cmax) {
+					size_t k = cells[j] - cmin;
+					if (newcell[k]) {
+						v[k] = values[j];
+						newcell[k] = false;
+					} else {
+						v[k] = std::max(v[k], values[j]);
+					}
+				} else {
+					cellcnt = j;
+					break;
+				}
+			}
+			if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i]))  return out;
+		}
+	} else if (fun == "prod") {
+		for (size_t i=0; i < out.bs.n; i++) {
+			double cmin = out.bs.row[i] * nc;
+			double cmax = (out.bs.row[i]+out.bs.nrows[i]) * nc - 1;
+			std::vector<double> v(out.bs.nrows[i] * out.ncol(), background);
+			std::vector<bool> newcell(out.bs.nrows[i] * out.ncol(), true);
+			for (size_t j=cellcnt; j<cells.size(); j++) {
+				if (narm && std::isnan(values[j])) continue;
+				if (cells[j] <= cmax) {
+					size_t k = cells[j] - cmin;
+					if (newcell[k]) {
+						v[k] = values[j];
+						newcell[k] = false;
+					} else {
+						v[k] *= values[j];
+					}
+				} else {
+					cellcnt = j;
+					break;
+				}
+			}
+			if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i]))  return out;
+		}
+	} else if (fun == "pa") {
+		for (size_t i=0; i < out.bs.n; i++) {
+			double cmin = out.bs.row[i] * nc;
+			double cmax = (out.bs.row[i]+out.bs.nrows[i]) * nc - 1;
+			std::vector<double> v(out.bs.nrows[i] * out.ncol(), background);
+			for (size_t j=cellcnt; j<cells.size(); j++) {
+				if (narm && std::isnan(values[j])) continue;
+				if (cells[j] <= cmax) {
+					size_t k = cells[j] - cmin;
+					v[k] = 1;
+				} else {
+					cellcnt = j;
+					break;
+				}
+			}
+			if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i]))  return out;
+		}
+	} else if (fun == "first") {
+		for (size_t i=0; i < out.bs.n; i++) {
+			double cmin = out.bs.row[i] * nc;
+			double cmax = (out.bs.row[i]+out.bs.nrows[i]) * nc - 1;
+			std::vector<double> v(out.bs.nrows[i] * out.ncol(), background);
+			std::vector<bool> newcell(out.bs.nrows[i] * out.ncol(), true);;
+			for (size_t j=cellcnt; j<cells.size(); j++) {
+				if (narm && std::isnan(values[j])) continue;
+				if (cells[j] <= cmax) {
+					size_t k = cells[j] - cmin;
+					if (newcell[k]) {
+						v[k] = values[j];
+						newcell[k] = false;
+					} 
+				} else {
+					cellcnt = j;
+					break;
+				}
 			}
 			if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i]))  return out;
 		}
@@ -98,10 +240,14 @@ SpatRaster SpatRaster::rasterizePoints(SpatVector x, std::string fun, std::vecto
 			double cmin = out.bs.row[i] * nc;
 			double cmax = (out.bs.row[i]+out.bs.nrows[i]) * nc - 1;
 			std::vector<double> v(out.bs.nrows[i] * out.ncol(), background);
-			for (size_t j=0; j<cells.size(); j++) {
-				if (cells[j] >= cmin && cells[j] <= cmax) {
+			for (size_t j=cellcnt; j<cells.size(); j++) {
+				if (narm && std::isnan(values[j])) continue;
+				if (cells[j] <= cmax) {
 					size_t k = cells[j] - cmin;
 					v[k] = values[j];
+				} else {
+					cellcnt = j;
+					break;
 				}
 			}
 			if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i]))  return out;
@@ -112,9 +258,22 @@ SpatRaster SpatRaster::rasterizePoints(SpatVector x, std::string fun, std::vecto
 }
 
 
+SpatRaster SpatRaster::rasterizePoints(SpatVector &x, std::string fun, std::vector<double> &values, bool narm, double background, SpatOptions &opt) {
+	if (values.empty()) {
+		values = std::vector<double>(x.nrow(), 1);
+	}
+	std::vector<std::vector<double>> pxy = x.coordinates();
+	return rasterizePoints(pxy[0], pxy[1], fun, values, narm, background, opt);
+}
+
+
 SpatRaster SpatRaster::rasterizeGeom(SpatVector x, std::string unit, std::string fun, SpatOptions &opt) {
 
-	if (x.type() != "points") {
+	if (x.type() == "points") {
+		std::vector<double> v;
+		return rasterizePoints(x, "count", v, false, 0.0, opt);
+
+	} else {	
 
 		SpatRaster out = geometry(1, false, false, false);
 		SpatOptions ops(opt);
@@ -201,8 +360,6 @@ SpatRaster SpatRaster::rasterizeGeom(SpatVector x, std::string unit, std::string
 		out.writeStop();
 		return(out);
 
-	} else {
-		return rasterizePoints(x, "count", {}, 0.0, opt);
 	}
 }
 
@@ -252,7 +409,7 @@ bool SpatRaster::getDSh(GDALDatasetH &rstDS, SpatRaster &out, std::string &filen
 	filename = opt.get_filename();
 	SpatOptions ops(opt);
 	ops.ncopies += 4;
-	if (filename == "") {
+	if (filename.empty()) {
 		if (canProcessInMemory(ops)) {
 			driver = "MEM";
 		} else {
@@ -263,7 +420,7 @@ bool SpatRaster::getDSh(GDALDatasetH &rstDS, SpatRaster &out, std::string &filen
 	} else {
 		driver = opt.get_filetype();
 		getGDALdriver(filename, driver);
-		if (driver == "") {
+		if (driver.empty()) {
 			out.setError("cannot guess file type from filename");
 			return false;
 		}
@@ -388,8 +545,8 @@ SpatRaster SpatRaster::rasterizeLyr(SpatVector x, double value, double backgroun
 
 
 SpatRaster SpatRaster::rasterize(SpatVector x, std::string field, std::vector<double> values,
-	double background, bool touches, bool add, bool weights, bool update, bool minmax, SpatOptions &opt) {
-
+	double background, bool touches, std::string fun, bool weights, bool update, bool minmax, SpatOptions &opt) {
+	
 	std::string gtype = x.type();
 	bool ispol = gtype == "polygons";
 	if (weights) update = false;
@@ -406,18 +563,20 @@ SpatRaster SpatRaster::rasterize(SpatVector x, std::string field, std::vector<do
 		//unsigned agy = 100;
 		wout = wout.disaggregate({agx, agy}, sopts);
 		double f = agx * agy;
-		wout = wout.rasterize(x, field, {1/f}, background, touches, add, false, false, false, sopts);
+		wout = wout.rasterize(x, field, {1/f}, background, touches, fun, false, false, false, sopts);
 		wout = wout.aggregate({agx, agy}, "sum", true, opt);
 		return wout;
 	}
 
+	bool add = fun == "sum";
+
 	SpatRaster out;
 	if ( !hasValues() ) update = false;
 	if (update) {
 		out = hardCopy(opt);
 	} else {
 		out = geometry(1);
-		if (field == "") {
+		if (field.empty()) {
 			out.setNames({"layer"});
 		} else {
 			out.setNames({field});
@@ -437,7 +596,7 @@ SpatRaster SpatRaster::rasterize(SpatVector x, std::string field, std::vector<do
 		out.addWarning("you cannot use 'sum' and 'touches' at the same time");
 	}
 
-	if (field != "") {
+	if (!field.empty()) {
 		int i = x.df.get_fieldindex(field);
 		if (i < 0) {
 			out.setError("field " + field + " not found");
@@ -588,7 +747,7 @@ SpatRaster SpatRaster::rasterize(SpatVector x, std::string field, std::vector<do
 		out = SpatRaster(filename, {-1}, {""}, {}, {});
 	} else {
 		std::string fname = opt.get_filename();
-		if ((fname != "") && (!update)) {
+		if (!fname.empty() && (!update)) {
 			out = out.writeRaster(opt);
 		}
 	}
@@ -621,17 +780,20 @@ std::vector<double> SpatRaster::rasterizeCells(SpatVector &v, bool touches, Spat
 
 	SpatRaster rc = r.crop(e, "out", false, ropt);
 	std::vector<double> feats(1, 1) ;
-    SpatRaster rcr = rc.rasterize(v, "", feats, NAN, touches, false, false, false, false, ropt);
+    SpatRaster rcr = rc.rasterize(v, "", feats, NAN, touches, "", false, false, false, ropt);
 	SpatVector pts = rcr.as_points(false, true, false, ropt);
 	std::vector<double> cells;
-	if (pts.size() == 0) {
+	if (pts.empty()) {
 		pts = v.as_points(false, true);
 		SpatDataFrame vd = pts.getGeometryDF();
 		std::vector<double> x = vd.getD(0);
 		std::vector<double> y = vd.getD(1);
-		cells = r.cellFromXY(x, y);
+		cells = r.cellFromXY(x, y);	
+		cells.erase(std::remove_if(cells.begin(), cells.end(),
+                    [](const double& value) { return std::isnan(value); }), cells.end());
+		std::sort( cells.begin(), cells.end() );
 		cells.erase(std::unique(cells.begin(), cells.end()), cells.end());
-		if (cells.size() == 0) {
+		if (cells.empty()) {
 			cells.resize(1, NAN);
 		}
 	} else {
@@ -639,6 +801,10 @@ std::vector<double> SpatRaster::rasterizeCells(SpatVector &v, bool touches, Spat
 		std::vector<double> x = vd.getD(0);
 		std::vector<double> y = vd.getD(1);
 		cells = r.cellFromXY(x, y);
+//		cells.erase(std::unique(cells.begin(), cells.end()), cells.end());
+		if (cells.empty()) {
+			cells.resize(1, NAN);
+		}
 	}
 	return cells;
 }
@@ -663,10 +829,10 @@ void SpatRaster::rasterizeCellsWeights(std::vector<double> &cells, std::vector<d
 		r = rc;
 	}
 	std::vector<double> feats;
-	r = r.rasterize(v, "", feats, NAN, false, false, true, false, false, ropt);
+	r = r.rasterize(v, "", feats, NAN, false, "", true, false, false, ropt);
 	std::vector<std::vector<double>> cv = r.cells_notna(ropt);
 
-	if (cv[0].size() == 0) {
+	if (cv[0].empty()) {
 		weights.resize(1);
 		weights[0] = NAN;
 		cells.resize(1);
@@ -692,10 +858,10 @@ void SpatRaster::rasterizeCellsExact(std::vector<double> &cells, std::vector<dou
 
 //	if (r.ncell() < 1000) {
 		std::vector<double> feats(1, 1) ;
-		r = r.rasterize(v, "", feats, NAN, true, false, false, false, false, ropt);
+		r = r.rasterize(v, "", feats, NAN, true, "", false, false, false, ropt);
 
 		SpatVector pts = r.as_points(true, true, false, ropt);
-		if (pts.size() == 0) {
+		if (pts.empty()) {
 			weights.resize(1);
 			weights[0] = NAN;
 			cells.resize(1);
